@@ -3,6 +3,24 @@
 """
 
 """
+2022/9/16
+control_version:0.7.0
+blcoklyFile_version:2.2.9
+blockly_version:2.2.2
+1、机器学习库重构（不过保留了之前的，以兼容以前版本的编程块）
+    影响到的库有machine_learning和unique
+2、blocklyFiles中的机器学习编程块做出部分改动
+3、增加颜色聚类的代码和块
+"""
+
+
+"""
+2022/9/3
+control_version:0.6.9
+blcoklyFile_version:2.2.8
+blockly_version:2.2.2
+修复了已知bug
+
 2022/8/27
 control_version:0.6.8
 blcoklyFile_version:2.2.8
