@@ -8,11 +8,14 @@ system_platform = sys.platform
 main_path = '/home/pi/class/'  # 读取和保存文件所用主文件夹
 if 'win' in system_platform:
     # 获取当前文件的位置
-    file_path = os.path.join(os.getcwd().split('blockly-electron')[0],'blockly-electron')
-    main_path = file_path + '\\resources\\assets\\class\\'
+    file_path = os.path.join(os.getcwd().split('blockly-electron')[0], 'blockly-electron')
+    if not os.path.exists(file_path):
+        if os.path.exists(os.path.join(os.getcwd(), "resources")):
+            file_path = os.getcwd()
+    main_path = os.path.join(file_path, 'resources', 'assets', 'class').replace("\\", "/")
 f_path = 'file_operation/'
-if not os.path.exists(main_path+f_path):
-    os.makedirs(main_path+f_path)
+if not os.path.exists(os.path.join(main_path,f_path)):
+    os.makedirs(os.path.join(main_path,f_path))
 
 
 class Common_file:
@@ -153,7 +156,7 @@ class CSV():
     赋值（f）为csv文件（example.csv)
     '''
 
-    def __init__(self, file_name:str,file_dir:str):
+    def __init__(self, file_name:str,file_dir:str,mode:str='r'):
         '''
 
         :param file_name: csv文件的名称，通常放置在 ../resources/assets/class/file_operation/内
